#include "trap.h"
#include "util.h"
#include "irc.h"
#include "log.h"
#include "main.h"
#include "config.h"

list<Trap> TrapList::traps;

Trap::Trap(const TrapAction action, const long timeout,
  const std::string & nick, const std::string & userhost,
  const std::string & reason)
{
  this->action = action;
  this->timeout = timeout;
  this->nick = nick;
  this->userhost = userhost;
  this->reason = reason;
}

bool
Trap::matches(const std::string & nick, const std::string & userhost) const
{
  return (MatchesMask(nick, this->nick) &&
    MatchesMask(userhost, this->userhost));
}

bool
Trap::exactMatch(const std::string & nick, const std::string & userhost) const
{
  return (Same(nick, this->nick) && Same(userhost, this->userhost));
}

TrapAction
Trap::getAction() const
{
  return this->action;
}

long
Trap::getTimeout() const
{
  return this->timeout;
}

std::string
Trap::getMask() const
{
  return (this->nick + "!" + this->userhost);
}

std::string
Trap::getReason() const
{
  return this->reason;
}

void
TrapList::split(const std::string & mask, std::string & nick,
  std::string & userhost)
{
  std::string::size_type bang = mask.find('!');
  std::string::size_type at = mask.find('@');

  if (bang != std::string::npos)
  {
    nick = mask.substr(0, bang);
    userhost = mask.substr(bang + 1, std::string::npos);
  }
  else if (at != std::string::npos)
  {
    nick = "*";
    userhost = mask;
  }
  else
  {
    nick = mask;
    userhost = "*@*";
  }
}

void
TrapList::add(const TrapAction action, const long timeout,
  const std::string & mask, const std::string & reason)
{
  std::string nick;
  std::string userhost;

  split(mask, nick, userhost);

  remove(nick, userhost);

  Trap trap(action, timeout, nick, userhost, reason);

  traps.push_back(trap);
}

void
TrapList::addEcho(const std::string & mask)
{
  add(TRAP_ECHO, 0, mask, "");
}

void
TrapList::addKill(const std::string & mask, const std::string & reason)
{
  add(TRAP_KILL, 0, mask, reason);
}

void
TrapList::addKline(const bool host, const std::string & mask,
  const std::string & reason, const long timeout = 0)
{
  add(host ? TRAP_KLINEHOST : TRAP_KLINE, timeout, mask, reason);
}

void
TrapList::addDline(const bool net, const std::string & mask,
  const std::string & reason)
{
  add(net ? TRAP_DLINENET : TRAP_DLINE, 0, mask, reason);
}

bool
TrapList::remove(const std::string & mask)
{
  std::string nick;
  std::string userhost;

  split(mask, nick, userhost);

  return remove(nick, userhost);
}

bool
TrapList::remove(const std::string & nick, const std::string & userhost)
{
  bool found = false;
  std::list<Trap>::iterator pos = traps.begin();
  while (pos != traps.end())
  {
    if (pos->exactMatch(nick, userhost))
    {
      pos = traps.erase(pos);
      found = true;
    }
    else
    {
      ++pos;
    }
  }
  return found;
}


void
TrapList::flush()
{
  traps.clear();
}

void
TrapList::match(const std::string & nick, const std::string & userhost,
  const std::string & ip)
{
  for (std::list<Trap>::iterator pos = traps.begin(); pos != traps.end(); ++pos)
  {
    if (!Config::IsOKHost(userhost, ip) && !Config::IsOper(userhost, ip) &&
      pos->matches(nick, userhost))
    {
      Log::Write(std::string("Trapped: ") + nick + "!" + userhost);
      switch (pos->getAction())
      {
	case TRAP_ECHO:
	  ::Status("Trapped: " + nick + "!" + userhost, SEND_OPERS_ONLY);
	  break;
	case TRAP_KILL:
	  IRC::Kill("Auto-Kill", nick, pos->getReason());
	  break;
	case TRAP_KLINE:
	  IRC::Kline("Auto-Kline", pos->getTimeout(), klineMask(userhost),
	    pos->getReason());
	  break;
	case TRAP_KLINEHOST:
	  {
	    std::string::size_type at = userhost.find('@');
	    if (at != std::string::npos)
	    {
	      std::string host = userhost.substr(at + 1, std::string::npos);
	      IRC::Kline("Auto-Kline", pos->getTimeout(), "*@" + host,
	        pos->getReason());
	    }
	    else
	    {
	      cerr << "Userhost is missing @ symbol!" << endl;
	    }
	  }
	  break;
	case TRAP_DLINE:
	  IRC::Dline("Auto-Dline", ip, pos->getReason());
	  break;
	case TRAP_DLINENET:
	  IRC::Dline("Auto-Dline", classCMask(ip), pos->getReason());
	  break;
	default:
	  cerr << "Unknown Trap type!" << endl;
	  break;
      }
    }
  }
}

void
TrapList::list(StrList & output)
{
  int count = 0;

  for (std::list<Trap>::iterator pos = traps.begin(); pos != traps.end(); ++pos)
  {
    std::string text;
    TrapAction action = pos->getAction();
    switch (action)
    {
      case TRAP_ECHO:
	text = "ECHO: " + pos->getMask();
	break;
      case TRAP_KILL:
	text = "KILL: " + pos->getMask() + " (" + pos->getReason() + ")";
	break;
      case TRAP_KLINE:
	if (pos->getTimeout() > 0)
	{
	  text = "KLINE: " + ULongToStr(pos->getTimeout()) + " " +
	    pos->getMask() + " (" + pos->getReason() + ")";
	}
	else
	{
	  text = "KLINE: " + pos->getMask() + " (" + pos->getReason() + ")";
	}
	break;
      case TRAP_KLINEHOST:
	if (pos->getTimeout() > 0)
	{
	  text = "KLINEHOST: " + ULongToStr(pos->getTimeout()) + " " +
	    pos->getMask() + " (" + pos->getReason() + ")";
	}
	else
	{
	  text = "KLINEHOST: " + pos->getMask() + " (" + pos->getReason() + ")";
	}
	break;
      case TRAP_DLINE:
	text = "DLINE: " + pos->getMask() + " (" + pos->getReason() + ")";
	break;
      case TRAP_DLINENET:
	text = "DLINENET: " + pos->getMask() + " (" + pos->getReason() + ")";
	break;
      default:
	text = "Unknown TRAP type!";
	break;
    }
    output.push_back(text);
    count++;
  }
  if (count > 0)
  {
    output.push_back("*** End of TRAP list.");
  }
  else
  {
    output.push_back("*** TRAP list is empty!");
  }
}

